# hangul_ends_with_consonant.py

from hangulpy.utils import is_hangul, HANGUL_BEGIN_UNICODE, CHOSUNG_LIST, JONGSUNG_LIST

def ends_with_consonant(char):
	"""
	주어진 문자가 자음으로 끝나는지 확인합니다.
	
	:param char: 확인할 문자
	:return: 자음으로 끝나면 True, 아니면 False
	"""
	if not is_hangul(char):
		return char in CHOSUNG_LIST

	# 한글 음절의 유니코드 값을 기준으로 종성 인덱스를 확인합니다.
	char_index = ord(char) - HANGUL_BEGIN_UNICODE
	jongsung_index = char_index % 28  # 28개의 종성

	return JONGSUNG_LIST[jongsung_index] != ''



# is_hangul.py
from hangulpy.utils import is_hangul as utils_is_hangul

def is_hangul(text, spaces=False):
    """
    입력된 값이 한글인지 확인합니다.

    :param text: 문자 또는 문자열
    :param spaces: True일 경우 띄어쓰기를 허용합니다.
    :return: 모든 문자가 한글이거나, spaces=True일 경우 한글 또는 띄어쓰기면 True, 그렇지 않으면 False
    """
    return utils_is_hangul(text, spaces)



# sort_hangul.py

from hangulpy.hangul_decompose import decompose_hangul_string

def sort_hangul(words):
    """
    한글 문자열을 초성, 중성, 종성을 기준으로 정렬합니다.

    :param words: 한글 문자열 리스트
    :return: 정렬된 한글 문자열 리스트
    """
    if not isinstance(words, list):
        raise ValueError("입력 값은 리스트여야 합니다. (Input must be a list of strings)")

    def hangul_key(word):
        # 각 글자를 초성, 중성, 종성으로 분해하여 정렬 키로 사용
        decomposed = decompose_hangul_string(word)
        # 분해된 결과를 튜플로 묶어서 반환
        return [(chosung, ''.join(jungsung), ''.join(jongsung)) for chosung, jungsung, jongsung in decomposed]

    return sorted(words, key=hangul_key)



import re
from hangulpy.hangul_decompose import decompose_hangul_string

def match_hangul_pattern(words, pattern):
    """
    주어진 단어 리스트에서 특정 초성, 중성, 종성 패턴에 매칭되는 단어를 찾습니다.

    :param words: 단어 리스트
    :param pattern: 초성, 중성, 종성 패턴 (예: '*ㅜ')
    :return: 패턴에 매칭되는 단어 리스트
    """
    # 패턴을 정규식으로 변환
    regex_pattern = re.compile(pattern.replace('*', '.*'))

    matched_words = []
    for word in words:
        decomposed = decompose_hangul_string(word)  # 한글 분해
        decomposed_flat = ''.join(
            chosung + ''.join(jungsung) + ''.join(jongsung)
            for chosung, jungsung, jongsung in decomposed
        )  # 초성, 중성, 종성을 문자열로 결합

        if regex_pattern.fullmatch(decomposed_flat):
            matched_words.append(word)

    return matched_words



# hangul_decompose.py

import warnings
from hangulpy.utils import (
    CHOSUNG_LIST, CHOSUNG_BASE, is_hangul, HANGUL_BEGIN_UNICODE,
    JUNGSUNG_LIST, JUNGSUNG_DECOMPOSE, JONGSUNG_LIST, JONGSUNG_DECOMPOSE, JUNGSUNG_BASE
)

def decompose_hangul_char(c):
    """
    주어진 한글 음절을 초성, 중성, 종성으로 분해합니다.

    Deprecated: 이 함수는 미래 버전에서 제거될 예정입니다.
    대신 decompose_hangul_string 함수를 사용하세요.

    :param c: 한글 음절 문자
    :return: (초성, 중성, 종성) 튜플
    """
    warnings.warn(
        "The decompose_hangul_char function will be removed in a future version. Use the decompose_hangul_string function. "
        "Check out https://github.com/gaon12/hangulpy/issues/10 for more information!",
        DeprecationWarning,
        stacklevel=2
    )

    if is_hangul(c):
        # 한글 음절의 유니코드 값을 기준으로 각 성분의 인덱스를 계산합니다.
        char_index = ord(c) - HANGUL_BEGIN_UNICODE
        chosung_index = char_index // CHOSUNG_BASE
        jungsung_index = (char_index % CHOSUNG_BASE) // JUNGSUNG_BASE
        jongsung_index = char_index % JUNGSUNG_BASE

        # 중성 및 종성 분해
        jungsung = JUNGSUNG_LIST[jungsung_index]
        jongsung = JONGSUNG_LIST[jongsung_index]

        jungsung_decomposed = JUNGSUNG_DECOMPOSE.get(jungsung, (jungsung,))
        jongsung_decomposed = JONGSUNG_DECOMPOSE.get(jongsung, (jongsung,))

        return (CHOSUNG_LIST[chosung_index], jungsung_decomposed, jongsung_decomposed)

    return (c, '', '')  # 한글이 아니면 그대로 반환

def decompose_hangul_string(s):
    """
    주어진 문자열의 각 한글 음절을 초성, 중성, 종성으로 분해하여 배열 형태로 반환합니다.

    :param s: 문자열
    :return: 각 한글 음절을 초성, 중성, 종성으로 분해한 결과를 포함하는 배열
    """
    result = []
    for char in s:
        if is_hangul(char):
            # 한글 음절의 유니코드 값을 기준으로 각 성분의 인덱스를 계산합니다.
            char_index = ord(char) - HANGUL_BEGIN_UNICODE
            chosung_index = char_index // CHOSUNG_BASE
            jungsung_index = (char_index % CHOSUNG_BASE) // JUNGSUNG_BASE
            jongsung_index = char_index % JUNGSUNG_BASE

            # 중성 및 종성 분해
            jungsung = JUNGSUNG_LIST[jungsung_index]
            jongsung = JONGSUNG_LIST[jongsung_index]

            jungsung_decomposed = JUNGSUNG_DECOMPOSE.get(jungsung, (jungsung,))
            jongsung_decomposed = JONGSUNG_DECOMPOSE.get(jongsung, (jongsung,))

            result.append((CHOSUNG_LIST[chosung_index], jungsung_decomposed, jongsung_decomposed))
        else:
            result.append((char, '', ''))  # 한글이 아니면 그대로 추가

    return result
